unidata = "http://www.unidata.ucar.edu/namespaces/thredds/InvCatalog/v1.0"
w3 = "http://www.w3.org/1999/xlink"
iso = "http://www.isotc211.org/2005/gmd"
global_bounding_box = [-360,-90,0,90]
No_inf = "No information"